using System;
using Autofac;
using FitConnect;
using FitConnect.Encryption;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using MockContainer;
using NUnit.Framework;

namespace BasicUnitTest;

public class SenderTests {
    private IContainer _container = null!;
    protected string clientId = null!;
    protected string clientSecret = null!;
    private string leikaKey = null!;
    private ILogger? logger;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        _container = Container.Create();
        logger = _container.Resolve<ILogger>();
        var settings = _container.Resolve<IFitConnectSettings>();
        clientId = settings.SenderClientId;
        clientSecret = settings.SenderClientSecret;
        leikaKey = settings.LeikaKey;
    }

    [SetUp]
    public void Setup() {
        logger = _container.Resolve<ILogger>();
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_ArgumentExceptionLeikaKeyInvalid() {
        Assert.Throws<ArgumentException>(() => {
            var sender = Client.GetSender(Container.Create());
            var outgoing = SubmissionBuilder
                .WithDestination(Guid.NewGuid().ToString())
                .WithServiceType("", "")
                .WithJsonData("")
                .WithAttachments(Array.Empty<Attachment>())
                .Build();


            sender.SendAsync(outgoing).Wait();
        })!.Message.Should().Be("Invalid leika key");
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServicesWithData() {
        var sender = Client.GetSender(Container.Create());

        var outgoing = SubmissionBuilder
            .WithDestination(Guid.NewGuid().ToString())
            .WithServiceType("", leikaKey)
            .WithJsonData("")
            .WithAttachments(Array.Empty<Attachment>())
            .Build();
        sender.SendAsync(outgoing).Wait();
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServicesWithXmlData() {
        var sender = new Sender(FitConnectEnvironment.Testing,
            clientId, clientSecret,
            Container.Create());

        var outgoing = SubmissionBuilder
            .WithDestination(Guid.NewGuid().ToString())
            .WithServiceType("", leikaKey)
            .WithXmlData(
                @"<?xml version=""1.0"" encoding=""UTF-8""?> <root> <child>value</child> </root> ")
            .WithAttachments(Array.Empty<Attachment>())
            .Build();
        sender.SendAsync(outgoing).Wait();
    }

    [Test]
    public void FluentSender_ShouldNotThrowAnError_MockingServicesWithoutData() {
        var sender = Client.GetSender(Container.Create());
        var outgoing = SubmissionBuilder
            .WithDestination(Guid.NewGuid().ToString())
            .WithServiceType("", leikaKey)
            .WithJsonData(@"{""data"":""value""}")
            .WithAttachments(Array.Empty<Attachment>())
            .Build();

        sender.SendAsync(outgoing).Wait();
    }

    [Test]
    public void VerifyMetaData_ValidData_Fine() {
        // Arrange
        var submission = new Submission();
        submission.Data = @"{""data"":""value""}";
        submission.ServiceType.Identifier = leikaKey;

        // Act
        var metadata = Sender.CreateMetadata(submission);

        // Assert
        var valid = JsonHelper.VerifyMetadata(metadata, null, logger).Result;

        valid.Should().BeTrue();
    }

    [Test]
    public void VerifyMetaData_MissingLeikaKey_ThrowsAnError() {
        // Arrange
        var submission = new Submission();
        submission.Data = @"{""data"":""value""}";

        // Act
        var metadata = Sender.CreateMetadata(submission);

        // Assert
        var valid = JsonHelper.VerifyMetadata(metadata, null, logger).Result;
        valid.Should().BeFalse();
    }
}
